# coding: iso-8859-2

# ##################
# # Import Modules #
# ##################

import datetime
import getopt
import operator
import os
import shutil
import sys
import stat
import time
import win32com.client

# #########################################
# # Define Global Constants And Variables #
# #########################################

# Script version
SCRIPT_VER =                 '1.5'
# Original working directory
DIR_SCRIPT =                 os.getcwd()
# Common Resources Directory
DIR_COMMON =                 str(os.path.split(DIR_SCRIPT)[0]) + "\\Common_Resources\\"
# Directory in which the project(s) are to be created
DIR_CURRENT =                sys.argv[1]
# Project directory structure creator script and file
SCRIPT_CREATE_STRUCTURE =    "F:\\External_Scripts\\Python\\File_and_Directory_Operations\\create-dir-structure.py"
DIR_STRUCTURE =              DIR_SCRIPT + "\\create-dir-structure-input-work-all-media.txt"
# Project creator file
FILE_CREATOR =               "project-creator.txt"
# Location of the external text editor
UTIL_TEXT_EDITOR =           DIR_COMMON + "paddy.exe"

os.chdir(DIR_CURRENT)

# Name of the script (without path)
SCRIPT_NAME =                str(sys.argv[0][sys.argv[0].rfind("\\")+1:])

# Indent of the usage sytax descriptions before arguments when requestng help
TAB_0 =                      7
# Indent of the usage sytax descriptions after arguments when requestng help
TAB_1 =                      20

MAN =                        [
                             ("NAME",            "Create projects in the current directory according to the input file's lines,\n" + str(" " * TAB_0) + "Create directory structure, copy templates and set up some required files for the work."),
                             ("VER",             SCRIPT_VER),
                             ("SYNOPSIS",        "[OPTION]... <input file>"),
                             ("DESCRIPTION",     ""),
                             ("AUTHOR",          "Written by Adam Pocs"),
                             ("REPORTING BUGS",  "Report bugs to royaljerry@gmail.com"),
                             ("COPYRIGHT",       "Copyright (C) 2011 Adam Pocs; this is free software.")
                             ]
OPTIONS_SHORT =              [
                             ("h",               "Print this message and exit."),
                             ("v",               "Verbose mode.")
                             ]

# Project list
PROJECTS =                   []

# Basic templates to copy
TEMPLATES_COPY =             [
                             ("H:\\Assets\\Project_1.5\\common-files\\swatches\\",                           "RESOURCES\\Templates\\swatches\\"),
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\as\\",                          "RESOURCES\\Projects\\flash\\as\\"),
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\assets\\",                      "RESOURCES\\Projects\\flash\\assets\\"),
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\doc\\",                         "RESOURCES\\Projects\\flash\\doc\\"),
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\fla\\",                         "RESOURCES\\Projects\\flash\\fla\\"),
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\release\\",                     "RESOURCES\\Projects\\flash\\release\\"),
                             ("H:\\Assets\\Project_1.5\\common-files\\etc\\Objects.mm",                      "RESOURCES\\Info\\generic\\Objects.mm"),
                             ("H:\\Assets\\Project_1.5\\common-files\\etc\\project-info.xls",                "RESOURCES\\Info\\admin\\__PROJECT_FOLDER__.xls"),
                             ("H:\\Assets\\Project_1.5\\common-files\\etc\\Colors.html",                     "RESOURCES\\Info\\generic\\Colors.html"),
                             ("H:\\Assets\\Project_1.5\\common-files\\web\\",                                "RESULT\\Web\\")
                             ]

# Templates with content to be updated according to the current project
TEMPLATES_DYN =              [
                             # desktop.ini
                             ("H:\\Assets\\Project_1.5\\common-files\\etc\\desktop-ini-template.txt",        "desktop.ini"),
                             # Temporary File
                             ("H:\\Assets\\Project_1.5\\common-files\\etc\\temp.txt",                        "RESOURCES\\ETC\\temp_text\\temp.txt"),
                             # ToDoList Project
                             ("H:\\Assets\\Project_1.5\\common-files\\etc\\bugtracker.tdl",                  "RESOURCES\\Info\\generic\\__PROJECT_FOLDER__.tdl"),
                             # Total Commander Tablist
                             ("H:\\Assets\\Project_1.5\\common-files\\etc\\total-commander-tablist.tab",     "RESOURCES\\ETC\\__PROJECT_FOLDER__.tab"),
                             # Total Commander Tablist (Client)
                             ("H:\\Assets\\Project_1.5\\common-files\\etc\\total-commander-tablist-client.tab", "RESOURCES\\ETC\\__PROJECT_FOLDER___[CLIENT].tab"),
                             # Total Commander Tablist (Current)
                             ("H:\\Assets\\Project_1.5\\common-files\\etc\\total-commander-tablist-current.tab", "RESOURCES\\ETC\\__PROJECT_FOLDER___[CURRENT].tab"),
                             # Total Commander Tablist (Flash)
                             ("H:\\Assets\\Project_1.5\\common-files\\etc\\total-commander-tablist-flash.tab", "RESOURCES\\ETC\\__PROJECT_FOLDER___[FLASH].tab"),
                             # Total Commander Tablist (InDesign)
                             ("H:\\Assets\\Project_1.5\\common-files\\etc\\total-commander-tablist-indesign.tab", "RESOURCES\\ETC\\__PROJECT_FOLDER___[INDESIGN].tab"),
                             # Flash AS __MUTATION__ script
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\as\\__MUTATION__\\mutation.as", "RESOURCES\\Projects\\flash\\as\\__MUTATION__\\mutation.as"),
                             # Flash AS common script #1
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\as\\common\\preloading.as",     "RESOURCES\\Projects\\flash\\as\\common\\preloading.as"),
                             # Flash AS common script #2
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\as\\common\\run.as",            "RESOURCES\\Projects\\flash\\as\\common\\run.as"),
                             # Flash AS common script #3
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\as\\common\\standard.as",       "RESOURCES\\Projects\\flash\\as\\common\\standard.as"),
                             # Flash AS common script #4
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\as\\common\\standard-ext.as",   "RESOURCES\\Projects\\flash\\as\\common\\standard-ext.as"),
                             # Flash AS common script #5
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\as\\common\\standard-m.as",     "RESOURCES\\Projects\\flash\\as\\common\\standard-m.as"),
                             # Flash AS common script #6
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\as\\common\\standard-num.as",   "RESOURCES\\Projects\\flash\\as\\common\\standard-num.as"),
                             # Flash AS common script #7
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\as\\common\\standard-str.as",   "RESOURCES\\Projects\\flash\\as\\common\\standard-str.as"),
                             # Flash AS common script #8
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\as\\common\\standard-xml.as",   "RESOURCES\\Projects\\flash\\as\\common\\standard-xml.as"),
                             # Flash Project File
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\__PROJECT_FOLDER__.flp",        "RESOURCES\\Projects\\flash\\__PROJECT_FOLDER__.flp"),
                             # Flash Project File (Publish)
                             ("H:\\Assets\\Project_1.5\\common-files\\flash\\__PROJECT_FOLDER___[PUBLISH].flp", "RESOURCES\\Projects\\flash\\__PROJECT_FOLDER___[PUBLISH].flp"),
                             # CFDG Graphics
                             ("H:\\Assets\\Project_1.5\\common-files\\cfdg\\graphics.cfdg",                  "RESOURCES\\Templates\\code\\cfdg\\graphics.cfdg"),
                             # CFDG Importer
                             ("H:\\Assets\\Project_1.5\\common-files\\cfdg\\importer.cfdg",                  "RESOURCES\\Templates\\code\\cfdg\\importer.cfdg"),
                             # Data + Calc
                             ("H:\\Assets\\Project_1.5\\common-files\\etc\\data+calc.spq",                   "RESOURCES\\Info\\generic\\__PROJECT_FOLDER__.spq")
                             ]

# Empty textfiles to be created
CREATE_EMPTY_FILES =         [
                             "RESOURCES\\ETC\\temp_text\\formatting.txt",
                             "RESOURCES\\Info\\generic\\ep-project.txt"
                             ]

# Predefined working directories with corresponding icons
PREDEF_DIR_ICONS =           [
                             ("G:\\WORK\\Official_Permanent\\",                                              "H:\\Assets\\Project_1.5\\common-files\\icons\\Official_Permanent.ico"),
                             ("G:\\WORK\\Official_Single\\",                                                 "H:\\Assets\\Project_1.5\\common-files\\icons\\Official_Single.ico"),
                             ("G:\\WORK\\Self_Permanent\\",                                                  "H:\\Assets\\Project_1.5\\common-files\\icons\\Self_Permanent.ico"),
                             ("G:\\WORK\\Self_Single\\",                                                     "H:\\Assets\\Project_1.5\\common-files\\icons\\Self_Single.ico"),
                             ("G:\\WORK\\SEP_Family\\",                                                      "H:\\Assets\\Project_1.5\\common-files\\icons\\SEP_Family.ico"),
                             ("G:\\WORK\\SEP_Generic\\",                                                     "H:\\Assets\\Project_1.5\\common-files\\icons\\SEP_Generic.ico"),
                             ("G:\\WORK\\Self_Permanent\\Typo\\Projects\\",                                  "H:\\Assets\\Project_1.5\\common-files\\icons\\Typo_Projects.ico"),
                             ("G:\\WORK\\Self_Permanent\\Typo\\Works\\",                                     "H:\\Assets\\Project_1.5\\common-files\\icons\\Typo_Works.ico")
                             ]

# Shortcuts to be created (descriptor, target, link)
CREATE_SHORTCUTS =           [
                             # Project Shortcuts
                             {'desc': 'p',       'target': "",                                               'link': "F:\\Shortcuts\\Projects\\"},
                             # Desktop
                             {'desc': 'd',       'target': "",                                               'link': "C:\\Users\\Royaljerry\\Desktop\\"},
                             # Adobe Illustrator Document Templates
                             {'desc': 'a',       'target': "RESOURCES\\Templates\\graphics\\",               'link': "C:\\Program Files (x86)\\Adobe\\Adobe Illustrator CS4\\Cool Extras\\en_GB\\Templates\\"}
                             ]

PROJECT_ICON_FILE =          "RESOURCES\\Info\\admin\\project.ico"
PROJECT_LOGFILE =            "RESOURCES\\Info\\admin\\project.log"
PROJECT_CREATOR =            "Adam Pcs (royaljerry@gmail.com)"

CHARS_NO_ACUTES =            {
                             'a':                '',
                             'e':                '',
                             'i':                '',
                             'o':                '',
                             'u':                '',
                             'A':                '',
                             'E':                '',
                             'I':                '',
                             'O':                '',
                             'U':                ''
                             }

CHARS_FOLDER_SAFE =          {
                             '_':                '\/:*?"<>| '
                             }

CHARS_LINK_SAFE =            {
                             '_':                '\/:*?"<>|'
                             }

CHARS_XML_ESCAPE =           {
                             '&quot;':           '"',
                             '&lt;':             '<',
                             '&gt;':             '>',
                             '&amp;':            '&'
                             }

# Print this message and exit
O_HELP =                     False
# Verbose mode
O_VERBOSE =                  False

# #############
# # Functions #
# #############

def printUsage(sName, man, oShort, indent0, indent1):
	# Init vars for the text to be printed
	cPrint = ""
	fChar = " "
	# Print sections and their descriptions
	for i in range(len(man)):
		nMan = man[i][0]
		if nMan == "NAME": dMan = sName + ": " + man[i][1]
		elif nMan == "SYNOPSIS": dMan = sName + " " + man[i][1]
		else: dMan = man[i][1]
		if nMan == "DESCRIPTION":
			cPrint += nMan
			# Print short descriptions
			for opt in range(len(oShort)):
				cPrint += "\n\n"
				cLine = str(fChar*TAB_0) + "-" + oShort[opt][0].ljust(indent1, " ") + oShort[opt][1]
				cPrint += cLine
		else:
			cPrint += nMan
			cPrint += "\n\n"
			cPrint += str(fChar*indent0) + dMan
		cPrint += "\n\n\n"
	print (cPrint)

def parseOptions():
	# Collect all possible short option names
	cOptsShort = ""
	for i in range(len(OPTIONS_SHORT)):
		if len(OPTIONS_SHORT[i][0]) == 1: cOptsShort += OPTIONS_SHORT[i][0] + ","
		else: cOptsShort += OPTIONS_SHORT[i][0][0:1] + ":,"
	cOptsShort = cOptsShort.rstrip(",")
	# Collect all possible long option names
	cOptsLong = []
	try:
		# Get current options
		opts, args = getopt.getopt(sys.argv[1:], cOptsShort, cOptsLong)
	except getopt.GetoptError:
		# If something went wrong, print error, help information and exit
		print ("\nSomething went wrong...\n")
		printUsage(str(globals()["SCRIPT_NAME"]), globals()["MAN"], globals()["OPTIONS_SHORT"], globals()["TAB_0"], globals()["TAB_1"])
		sys.exit(2)

	if O_VERBOSE: print ("")

	# Parse short options
	for op, ar in opts:
		if op == "-h": globals()["O_HELP"] =         True
		if op == "-v":
			globals()["O_VERBOSE"] =                   True
			print ("\nEntering Verbose Mode...\n")

def changeChars(inputString, charsDict):
	for oChar, cChars in charsDict.iteritems():
		for c in cChars:
			inputString = inputString.replace(c, oChar)
	return (inputString)

def setPermFile(cFile, cMode):
	hideFile = os.popen('attrib +' + cMode + ' ' + cFile)
	hideFileR = hideFile.read()
	hideFile.close()
	

# Create (but don't copy!) a textfile based on a template,
# and update its content based on the given markers
#
# pRoot:           Absolute path to the project root [String]
# tName:           Absolute path to the template file [String]
# cName:           Path to the current template name according to the project root (Cannot contain backward relative paths.) [String]
# pMarkers:        Marker list, these values should be updated in the current templates [List]
def updateContent(pRoot, tName, cName, pMarkers):
	template_obj =                                 open(tName, "r")
	template_lines =                               template_obj.readlines()
	template_obj.close()
	current_name =                                 pRoot + cName
	current_obj =                                  open(current_name, "w")
	for cLine in template_lines:
		template_current_line =                      cLine
		for cMarker in pMarkers:
			cFrom =                                    cMarker[0]
			cTo =                                      cMarker[1]
			template_current_line =                    template_current_line.replace(cFrom, cTo)
		current_obj.writelines(template_current_line)
	current_obj.close()

def getDateAndTimeFormatted():
	lTime =                                        time.localtime()
	rTime =                                        ""
	for i in range(6):
		if len(str(lTime[i])) < 2:                   rTime += str(lTime[i]).zfill(2) + "-"
		else:                                        rTime += str(lTime[i]) + "-"
	rTime =                                        rTime.rstrip("-")
	cDate =                                        rTime[: 10].replace("-", "/")
	cTime =                                        rTime[11 :].replace("-", ":")
	result =                                       cDate + " at " + cTime + " [CET]"
	return(result)

def main():

	# Parse options
	parseOptions()

	if O_VERBOSE:
		print ("Print this message and exit (-h): ".ljust(70, ".") + str(O_HELP))
		print ("Verbose mode (-v): ".ljust(70, ".") + str(O_VERBOSE))

	# Exit if help requested
	if O_HELP:
		print("")
		printUsage(str(globals()["SCRIPT_NAME"]), globals()["MAN"], globals()["OPTIONS_SHORT"], globals()["TAB_0"], globals()["TAB_1"])
		sys.exit(0)

	print("\n")

	# Create creator file, and open it in the text editor
	creator_file_name =                            globals()["DIR_CURRENT"] + globals()["FILE_CREATOR"]
	creator_file =                                 open(creator_file_name, "w")
	creator_file.writelines("####################################################\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("# PROJECT CREATOR PYTHON SCRIPT v" + SCRIPT_VER + "               #\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("# Adam Pcs, 2011 (royaljerry@gmail.com)           #\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("# USAGE                                            #\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("# Enter project definitions(s) below,              #\n")
	creator_file.writelines("# one definitions in each line.                    #\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("# Separate attributes with tabulator.              #\n")
	creator_file.writelines("# Ordering of attributes is disregarded.           #\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("# If you are ready, save and close it.             #\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("# ATTRIBUTE SYNTAX                                 #\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("# [Project Name]                                   #\n")
	creator_file.writelines("#     This is treated a valid Project Name if      #\n")
	creator_file.writelines("#     it doesn't start with a '-' (dash).          #\n")
	creator_file.writelines("# -i: [Path to icon file]                          #\n")
	creator_file.writelines("#     If not set, and the project is created in    #\n")
	creator_file.writelines("#     one of the predefined default working        #\n")
	creator_file.writelines("#     directories, this defaults to the icon       #\n")
	creator_file.writelines("#     of the corresponding working directory.      #\n")
	creator_file.writelines("# -s: [Shorcut descriptors]                        #\n")
	creator_file.writelines("#     One or more item(s) from the predefined      #\n")
	creator_file.writelines("#     shortcuts (see CREATE_SHORTCUTS variable)    #\n")
	creator_file.writelines("#     for details. Defaults to 'pda'.              #\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("# Empty lines, lines starting with '#' and         #\n")
	creator_file.writelines("# surrounding whitespaces will be disregarded.     #\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("# EXAMPLES                                         #\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("# myProject                                        #\n")
	creator_file.writelines("# myProject with spaces in its name                #\n")
	creator_file.writelines("# myProject -i:C:\\path\\to\\icon.ico                 #\n")
	creator_file.writelines("# myProject -s:dp                                  #\n")
	creator_file.writelines("# myProject -i:C:\\myIconfile.ico -s:ad             #\n")
	creator_file.writelines("#                                                  #\n")
	creator_file.writelines("####################################################\n")
	creator_file.writelines("\n")
	creator_file.close()
	command =                                      globals()["UTIL_TEXT_EDITOR"] + " " + creator_file_name
	os.system(command)

	# Get Timestamp
	project_timestamp =                            getDateAndTimeFormatted()

	# Parse creator file

	# Open file, get lines and close it
	if O_VERBOSE: print("")
	cFile_obj =                                    open(creator_file_name, "r")
	pLines =                                       cFile_obj.readlines()
	cFile_obj.close()

	# Get parameters from each line
	for cLine in pLines:
		# Exclude comments and empty lines
		if (cLine.lstrip()[0 : 1] != "#") and (cLine.lstrip().rstrip() != ""):
			cLineSep =                                 cLine.rstrip("\n").lstrip().rstrip().split("\t")
			cLineAttr =                                {}
			for cPart in cLineSep:
				cPartS =                                 cPart.lstrip().rstrip()
				# Icon
				if cPartS[0 : 2] == "-i":                cLineAttr['icon'] = cPartS[3 : ]
				# Shortcuts
				elif cPartS[0 : 2] == "-s":              cLineAttr['shortcuts'] = cPartS[3 : ]
				# Name
				elif cPartS != "" and cPartS[0 : 1] != "-": cLineAttr['name'] = cPartS
			PROJECTS.append(cLineAttr)

	# Loop thru' the projects
	for cProject in PROJECTS:
		# Skip project if it hasn't any names
		if 'name' in cProject:
			# Set project parameters with defaults

			# Name and name-related variables
			project_name =                             cProject['name']
			project_folder =                           changeChars(changeChars(project_name, globals()["CHARS_FOLDER_SAFE"]), globals()["CHARS_NO_ACUTES"])
			project_root =                             globals()["DIR_CURRENT"] + project_folder + "\\"

			# Icon
			project_icon =                             'default'
			# User-defined
			if 'icon' in cProject and os.path.isfile(cProject['icon']): project_icon = cProject['icon']
			# Default guessed from the current work directory
			else:
				for cDir in PREDEF_DIR_ICONS:
					if project_root.find(cDir[0]) != -1 and os.path.isfile(cDir[1]): project_icon = cDir[1]

			# Shortcuts
			if 'shortcuts' in cProject: project_shortcuts = cProject['shortcuts']
			else: project_shortcuts = 'pda'

			if O_VERBOSE: print("Creating project " + project_name)

			# Set marker values
			MARKERS =              [
                             ("__PROJECT_NAME__",                                                            project_name),
                             ("__PROJECT_NAME_XML__",                                                        changeChars(project_name, globals()["CHARS_XML_ESCAPE"])),
                             ("__PROJECT_FOLDER__",                                                          project_folder),
                             ("__PROJECT_ROOT__",                                                            project_root),
                             ("__TODOLIST_PATH__",                                                           project_root + "RESOURCES\\Info\\generic\\" + project_folder + ".tdl"),
                             ("__YEAR__",                                                                    str(time.localtime()[0])),
                             ("__DESKTOP_INI_ICONFILE__",                                                    PROJECT_ICON_FILE),
                             ("__DESKTOP_INI_INFOTIP__",                                                     "Project \"" + project_name + "\" [Created: " + project_timestamp + "]")
                             ]

			# Create directory structure
			if O_VERBOSE: print("Creating directory " + project_folder + " and project directory structure in it.")
			# Create the project root directory
			os.mkdir(project_folder)
			# Go into the created directory
			os.chdir(project_folder)
			# Create directory structure in the project root directory
			command =                                  globals()["SCRIPT_CREATE_STRUCTURE"] + " " + project_root + " " + globals()["DIR_STRUCTURE"]
			os.system(command)

			# Copy templates
			if O_VERBOSE: print("Copying templates into the project's directories.")
			for i in range(len(globals()["TEMPLATES_COPY"])):
				template_from =                          globals()["TEMPLATES_COPY"][i][0]
				template_to =                            project_root + globals()["TEMPLATES_COPY"][i][1].replace("__PROJECT_FOLDER__", project_folder)
				# If the template is a file, simply copy it to the desired location
				if os.path.isfile(template_from):
					shutil.copyfile(template_from, template_to)
				# If the template is a directory, remove the created one, and copy entire template directory structure
				else:
					shutil.rmtree(template_to)
					shutil.copytree(template_from, template_to)

			# Create empty textfiles
			if O_VERBOSE: print("Creating empty textfiles in the project's directories.")
			for i in range(len(globals()["CREATE_EMPTY_FILES"])):
				cFile =                                  project_root + globals()["CREATE_EMPTY_FILES"][i]
				cFile_obj =                              open(cFile, "w")
				cFile_obj.close()

			# Update contents of dynamic templates
			if O_VERBOSE: print("Updating contents of dynamic templates.")
			for cTemplate in globals()["TEMPLATES_DYN"]:
				updateContent(
					project_root,
					cTemplate[0],
					cTemplate[1].replace("__PROJECT_FOLDER__", project_folder),
					MARKERS
				)

			# Create project icon
			if project_icon != 'default': shutil.copyfile(project_icon, project_root + PROJECT_ICON_FILE)

			# Set project folder and desktop.ini permissions
			os.chmod(project_root, stat.S_IREAD)
			setPermFile(project_root + "desktop.ini", 'r')
			setPermFile(project_root + "desktop.ini", 'h')

			# Create project shortcuts
			if O_VERBOSE: print("Creating project shortcut.")
			for cShortcut in CREATE_SHORTCUTS:
				if project_shortcuts.find(cShortcut['desc']) != -1:
					shell =                                win32com.client.Dispatch("WScript.Shell")
					shortcutTo =                           cShortcut['link'] + changeChars(project_name, globals()["CHARS_LINK_SAFE"]) + ".lnk"
					shortcut =                             shell.CreateShortCut(shortcutTo)
					shortcut.Targetpath =                  project_root + cShortcut['target']
					shortcut.save()

			# Write project start info into the logfile
			if O_VERBOSE: print("Writing project start info into the logfile.")
			project_logfile_name =                     project_root + globals()["PROJECT_LOGFILE"]
			project_logfile =                          open(project_logfile_name, "w")
			project_logfile.writelines("Project name: " + project_name + "\n")
			project_logfile.writelines("Project location: " + project_root + "\n")
			project_logfile.writelines("Project created: " + project_timestamp + "\n")
			project_logfile.writelines("Project creator: " + globals()["PROJECT_CREATOR"] + "\n")
			project_logfile.writelines("\n")
			project_logfile.writelines("Shortcuts created outside the project directory:\n")
			project_logfile.writelines("\n")
			for cShortcut in CREATE_SHORTCUTS:
				if project_shortcuts.find(cShortcut['desc']) != -1:
					cLink = cShortcut['link'] + changeChars(project_name, globals()["CHARS_LINK_SAFE"]) + ".lnk"
					cTarget = project_root + cShortcut['target']
					project_logfile.writelines("Link \"" + cLink + "\" pointing to \"" + cTarget + "\"\n")
			project_logfile.writelines("\n")
			project_logfile.writelines("--------------------------------------------------\n")
			project_logfile.writelines("This project was created with Project Creator Python Script v" + SCRIPT_VER + " written by Adam Pcs [royaljerry@gmail.com]\n")

			# Get back to the working directory
			os.chdir("..")

	# Remove creator file
	os.remove(creator_file_name)

	# Wait for keypress if verbose mode is active
	if O_VERBOSE:
		print("Done.")
		raw_input()

# ########
# # MAIN #
# ########

main ()
